@@ -40,6 +40,7 @@
 #include "rtcom-eventlogger-ui/rtcom-log-search-bar.h"
 #include <gconf/gconf.h>
 #include <gconf/gconf-client.h>
+#include <stdint.h>
 #include "main.h"
 #include "time.h"
 #include "dbus/dbus.h"
@@ -48,8 +49,39 @@
 #include "settings.h"
 #include "filters.h"
 
+#define CSD_CALL_BUS_NAME	"com.nokia.csd.Call"
+#define CSD_CALL_INTERFACE	"com.nokia.csd.Call"
+#define CSD_CALL_INSTANCE	"com.nokia.csd.Call.Instance"
+#define CSD_CALL_CONFERENCE	"com.nokia.csd.Call.Conference"
+#define CSD_CALL_PATH		"/com/nokia/csd/call"
 
 
+static time_t date_to_epoch(gint year, gint month, gint day, gboolean start_of_day)
+{
+	time_t t;
+	struct tm time_str;
+	time_str.tm_year = year - 1900;
+	time_str.tm_mon = month;
+	time_str.tm_mday = day;
+	if(start_of_day)
+	{
+		time_str.tm_hour = 0;
+		time_str.tm_min = 0;
+		time_str.tm_sec = 1;
+	}
+	else
+	{
+		time_str.tm_hour = 23;
+		time_str.tm_min = 59;
+		time_str.tm_sec = 59;
+	}
+	time_str.tm_isdst = 0;
+
+	t = mktime(&time_str);
+	g_debug("time is %d", t);
+	return t;
+}
+
 static void
 get_time_string (
     gchar     *time_str,
@@ -95,6 +127,95 @@ get_time_string (
     }
 }
 
+
+
+static int send_method_call(const char *dest, const char *path,
+				const char *interface, const char *method,
+				DBusPendingCallNotifyFunction cb,
+				void *user_data, int type, ...)
+{
+	/*
+	 * ret = send_method_call(CSD_CALL_BUS_NAME, CSD_CALL_PATH,
+				CSD_CALL_INTERFACE, "CreateWith",
+				NULL, NULL,
+				DBUS_TYPE_STRING, &number,
+				DBUS_TYPE_UINT32, &flags,
+				DBUS_TYPE_INVALID);
+
+#define CSD_CALL_BUS_NAME	"com.nokia.csd.Call"
+#define CSD_CALL_INTERFACE	"com.nokia.csd.Call"
+#define CSD_CALL_INSTANCE	"com.nokia.csd.Call.Instance"
+#define CSD_CALL_CONFERENCE	"com.nokia.csd.Call.Conference"
+#define CSD_CALL_PATH		"/com/nokia/csd/call"
+	 */
+	DBusError * dberror = NULL;
+	DBusConnection * connection = dbus_bus_get(DBUS_BUS_SYSTEM, dberror);
+	if(dberror != NULL)
+	{
+		printf("dbus error\n");
+		printf("%s", dberror->message);
+		printf("\n");
+	}
+	DBusMessage *msg;
+	DBusPendingCall *call;
+	va_list args;
+
+	msg = dbus_message_new_method_call(dest, path, interface, method);
+	if (!msg) {
+		printf("unable to create dbus message");
+		error("Unable to allocate new D-Bus %s message", method);
+
+		return -1;
+	}
+
+	va_start(args, type);
+
+	if (!dbus_message_append_args_valist(msg, type, args)) {
+		dbus_message_unref(msg);
+		va_end(args);
+		return -1;
+	}
+
+	va_end(args);
+
+	if (!cb) {
+		printf("dbus send\n");
+		dbus_connection_send(connection, msg, NULL);
+		return 0;
+	}
+
+	if (!dbus_connection_send_with_reply(connection, msg, &call, -1)) {
+		error("Sending %s failed", method);
+		dbus_message_unref(msg);
+		return -1;
+	}
+
+	dbus_pending_call_set_notify(call, cb, user_data, NULL);
+	dbus_pending_call_unref(call);
+	dbus_message_unref(msg);
+	dbus_connection_unref(connection);
+	connection = NULL;
+
+	return 0;
+}
+
+static void make_call(GtkButton* button, gpointer data)
+{
+
+	int ret;
+	uint32_t flag = 0;
+	char * number = data;
+	printf("make call to %s", number);
+	printf("\n");
+
+	ret = send_method_call(CSD_CALL_BUS_NAME, CSD_CALL_PATH,
+					CSD_CALL_INTERFACE, "CreateWith",
+					NULL, NULL,
+					DBUS_TYPE_STRING, &number,
+					DBUS_TYPE_INT32, &flag,
+					DBUS_TYPE_INVALID);
+}
+
 static void delete_record(GtkButton* button, gpointer data)
 {
 	g_debug("delete record...");
@@ -223,6 +344,7 @@ void row_activated(GtkTreeView *tree_view, GtkTreePath *path,
 
 
 	delete_button = gtk_button_new_with_label("Delete");
+	hildon_gtk_widget_set_theme_size(delete_button, HILDON_SIZE_FINGER_HEIGHT);
 	g_signal_connect(
 	          G_OBJECT(delete_button),
 	          "clicked",
@@ -231,13 +353,15 @@ void row_activated(GtkTreeView *tree_view, GtkTreePath *path,
 	gtk_box_pack_start(GTK_BOX(button_box), delete_button, FALSE, FALSE, 0);
 	gtk_widget_show(delete_button);
 
-	call_button = gtk_button_new_with_label("Call");
-	/*g_signal_connect(
-	          G_OBJECT(all_button),
+	/*call_button = gtk_button_new_with_label(" Call ");
+	hildon_gtk_widget_set_theme_size(call_button, HILDON_SIZE_FINGER_HEIGHT);
+	g_signal_connect(
+	          G_OBJECT(call_button),
 	          "clicked",
-	          G_CALLBACK(delete_record),
-	          event_id);*/
-	/*gtk_box_pack_start(GTK_BOX(button_box), call_button, FALSE, FALSE, 0);
+	          G_CALLBACK(make_call),
+	          remote_account);
+
+	gtk_box_pack_start(GTK_BOX(button_box), call_button, FALSE, FALSE, 0);
 	gtk_widget_show(call_button);*/
 
 	contact_button = gtk_button_new_with_label("Contact");
@@ -282,8 +406,199 @@ static void button_clicked (GtkButton* button, gpointer data)
     gtk_main_quit();
 }
 
+static void toggle_date_filter (GtkButton* button, gpointer data)
+{
+	AppData *appdata = data;
+
+	if(hildon_check_button_get_active(HILDON_CHECK_BUTTON(button)))
+	{
+		g_debug("enabling date filter\n");
+		appdata->filter_by_date = TRUE;
+		return;
+	}
+
+	g_debug("disabling date filter\n");
+	appdata->filter_by_date = FALSE;
+
+}
 
 
+static void start_date_changed (HildonPickerButton* button, gpointer data)
+{
+	AppData *appdata = data;
+	gint day = 0;
+	gint month = 0;
+	gint year = 0;
+
+	hildon_date_button_get_date (HILDON_DATE_BUTTON(button), &year, &month, &day);
+	appdata->start_day = day;
+	appdata->start_month = month;
+	appdata->start_year = year;
+	g_debug("year %d, month %d, day %d", year, month, day);
+
+}
+
+static void end_date_changed (HildonPickerButton* button, gpointer data)
+{
+	AppData *appdata = data;
+	gint day = 0;
+	gint month = 0;
+	gint year = 0;
+
+	hildon_date_button_get_date (HILDON_DATE_BUTTON(button), &year, &month, &day);
+	appdata->end_day = day;
+	appdata->end_month = month;
+	appdata->end_year = year;
+	g_debug("year %d, month %d, day %d", year, month, day);
+
+}
+
+static void date_dialog_response (GtkDialog *dialog,
+                                     gint response_id, gpointer user_data)
+{
+	AppData *appdata = user_data;
+	time_t end_date;
+	time_t start_date;
+	if(appdata->filter_by_date)
+	{
+		g_debug("we will filter by date");
+		start_date = date_to_epoch(appdata->start_year, appdata->start_month, appdata->start_day, TRUE);
+
+		if(appdata->end_year != 0)
+			end_date = date_to_epoch(appdata->end_year, appdata->end_month, appdata->end_day, FALSE);
+		else
+			end_date = time(NULL);
+
+		g_debug("start date is %d", start_date);
+		g_debug("end date is %d", end_date);
+
+		if(start_date > end_date)
+		{
+			g_debug("start date greater than end date");
+			GtkWidget *banner = hildon_banner_show_information(GTK_WIDGET(appdata->mainWindow), NULL,
+					"Start Date must be before End date. Disabling filter");
+			appdata->filter_by_date = FALSE;
+			gtk_widget_destroy(GTK_WIDGET(dialog));
+			return;
+		}
+		else
+		{
+			g_debug("end date greater than start date so can continue");
+			appdata->start_date = (int)start_date;
+			appdata->end_date = (int)end_date;
+			filter_by_date(appdata);
+
+		}
+	}
+	else
+	{
+		g_debug("we won't filter by date");
+		filter_by_date(appdata);
+	}
+
+	gtk_widget_destroy(GTK_WIDGET(dialog));
+}
+
+static void date_dialog(GtkButton* button, gpointer data)
+{
+    GtkWidget *dialog;
+    GtkWidget *box;
+
+    GtkWidget *date_toggle_button;
+    GtkWidget *start_date_button;
+    GtkWidget *end_date_button;
+
+    AppData *appdata = data;
+
+
+
+    dialog = gtk_dialog_new_with_buttons("Date",
+			appdata->mainWindow,
+			GTK_DIALOG_MODAL,
+			GTK_STOCK_OK,
+			GTK_RESPONSE_OK,
+			NULL);
+
+    box = GTK_DIALOG(dialog)->vbox;
+
+
+
+    start_date_button = hildon_date_button_new(HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+    hildon_button_set_title(HILDON_BUTTON(start_date_button), "Start Date");
+    if(appdata->start_year != 0)
+    {
+    	hildon_date_button_set_date(HILDON_DATE_BUTTON(start_date_button),
+    			appdata->start_year, appdata->start_month, appdata->start_day);
+    }
+    else
+    {
+    	hildon_date_button_get_date (HILDON_DATE_BUTTON(start_date_button), &appdata->start_year,
+    			&appdata->start_month, &appdata->start_day);
+    }
+
+    g_signal_connect(
+        	G_OBJECT(start_date_button),
+            "value-changed",
+            G_CALLBACK(start_date_changed),
+            appdata);
+    gtk_widget_show(start_date_button);
+
+    end_date_button = hildon_date_button_new(HILDON_SIZE_FINGER_HEIGHT, HILDON_BUTTON_ARRANGEMENT_VERTICAL);
+    hildon_button_set_title(HILDON_BUTTON(end_date_button), "End Date");
+    if(appdata->end_year != 0)
+    {
+    	hildon_date_button_set_date(HILDON_DATE_BUTTON(end_date_button),
+    			appdata->end_year, appdata->end_month, appdata->end_day);
+    }
+    else
+    {
+     	hildon_date_button_get_date (HILDON_DATE_BUTTON(end_date_button), &appdata->end_year,
+     			&appdata->end_month, &appdata->end_day);
+    }
+
+    g_signal_connect(
+            G_OBJECT(end_date_button),
+            "value-changed",
+            G_CALLBACK(end_date_changed),
+            appdata);
+    gtk_widget_show(end_date_button);
+
+    date_toggle_button = hildon_check_button_new(HILDON_SIZE_AUTO );
+    gtk_button_set_label(GTK_BUTTON(date_toggle_button), "Filter By Date");
+
+    if(appdata->filter_by_date)
+    	hildon_check_button_set_active(HILDON_CHECK_BUTTON(date_toggle_button), TRUE);
+    else
+    	hildon_check_button_set_active(HILDON_CHECK_BUTTON(date_toggle_button), FALSE);
+
+    hildon_gtk_widget_set_theme_size(date_toggle_button, HILDON_SIZE_FINGER_HEIGHT);
+    gtk_widget_show(date_toggle_button);
+
+    g_signal_connect(
+    		G_OBJECT(date_toggle_button),
+            "toggled",
+            G_CALLBACK(toggle_date_filter),
+            appdata);
+
+    gtk_box_pack_start (GTK_BOX (box), date_toggle_button, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (box), start_date_button, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (box), end_date_button, FALSE, FALSE, 0);
+
+
+
+
+    /*GtkWidget *content_area = GTK_DIALOG(dialog)->vbox;
+    GtkWidget *content_widget = settings_widget_create(GTK_WINDOW(dialog));*/
+
+    /* Add the widget to the dialog
+    gtk_box_pack_start(GTK_BOX(content_area), content_widget, TRUE, TRUE, 10);*/
+
+    /* When a button (ok/cancel/etc.) is clicked or the dialog is closed - destroy it */
+    g_signal_connect(dialog, "response", G_CALLBACK(date_dialog_response), appdata);
+
+    gtk_dialog_run(GTK_DIALOG(dialog));
+
+}
 
 
 
@@ -369,14 +684,6 @@ static void call_type_dialog(GtkButton* button, gpointer data)
     gtk_box_pack_start (GTK_BOX (box), gsm_button, FALSE, FALSE, 0);
 
 
-
-    /*GtkWidget *content_area = GTK_DIALOG(dialog)->vbox;
-    GtkWidget *content_widget = settings_widget_create(GTK_WINDOW(dialog));*/
-
-    /* Add the widget to the dialog
-    gtk_box_pack_start(GTK_BOX(content_area), content_widget, TRUE, TRUE, 10);*/
-
-    /* When a button (ok/cancel/etc.) is clicked or the dialog is closed - destroy it */
     g_signal_connect(dialog, "response", G_CALLBACK(gtk_widget_destroy), NULL);
 
     gtk_dialog_run(GTK_DIALOG(dialog));
@@ -415,10 +722,13 @@ some_page_func (GtkNotebook *nb,
 			char *pstr = limit_s;
 			g_debug("checking int");
 			while (*pstr) {
-			    if (!g_ascii_isdigit(*pstr))
+			    if (!g_ascii_isdigit(*pstr) )
 			    {
-			    	g_debug("false");
-			    	return FALSE;
+			    	if(*pstr != '-')
+			    	{
+			    		g_debug("false %s", pstr);
+			    		return FALSE;
+			    	}
 			    }
 			    pstr++;
 			}
@@ -474,6 +784,7 @@ create_settings_wizard(AppData *appdata)
 	g_debug("creating settings...");
 	GtkWidget *wizard, *notebook;
 	GtkWidget *limit_label;
+	GtkWidget *desc_label;
 	GtkWidget *limit_entry;
 	GtkWidget *all_button, *voip_button, *gsm_button;
 	GtkWidget *main_vbox;
@@ -492,6 +803,7 @@ create_settings_wizard(AppData *appdata)
 	 * Setup the limit page
 	 */
 	limit_label = gtk_label_new ("Limit");
+	desc_label = gtk_label_new ("Number of calls to display.\n-1 for No Limit");
 
 	limit_entry = hildon_entry_new (HILDON_SIZE_AUTO);
 	g_object_set (G_OBJECT (limit_entry), "hildon-input-mode", HILDON_GTK_INPUT_MODE_NUMERIC, NULL);
@@ -509,6 +821,7 @@ create_settings_wizard(AppData *appdata)
 
 	gtk_box_pack_start (GTK_BOX (main_vbox), limit_label, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (main_vbox), limit_entry, FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (main_vbox), desc_label, FALSE, FALSE, 0);
 
 	/*
 	 * default type page
@@ -639,6 +952,14 @@ int main( int argc, char* argv[] )
 	AppData appdata;
 	appdata.static_group_by = RTCOM_EL_QUERY_GROUP_BY_NONE;
 	appdata.showing_details = FALSE;
+	appdata.filter_by_date = FALSE;
+	appdata.start_day = 0;
+	appdata.start_month = 0;
+	appdata.start_year = 0;
+	appdata.end_day = 0;
+	appdata.end_month = 0;
+	appdata.end_year = 0;
+
 
 
     osso_context_t *osso_cont;
@@ -653,6 +974,7 @@ int main( int argc, char* argv[] )
 	GtkWidget * missed_button = NULL;
     GtkWidget * all_button = NULL;
     GtkWidget * call_type_button = NULL;
+    GtkWidget * date_button = NULL;
     GtkWidget * settings_button = NULL;
     HildonAppMenu *menu;
 
@@ -710,6 +1032,8 @@ int main( int argc, char* argv[] )
 
     rtcom_log_model_populate(appdata.log_model, services);
     */
+    GtkWidget *banner = hildon_banner_show_information(GTK_WIDGET(appdata.mainWindow), NULL,
+    					"Loading calls may take time. Please be patient");
     populate_calls_default(&appdata);
 
     rtcom_log_search_bar_set_model(
@@ -766,7 +1090,7 @@ int main( int argc, char* argv[] )
     gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (all_button), FALSE);
 
     dialed_button = hildon_gtk_radio_button_new_from_widget (HILDON_SIZE_AUTO, GTK_RADIO_BUTTON(all_button));
-    gtk_button_set_label (GTK_BUTTON (dialed_button), "Dialled");
+    gtk_button_set_label (GTK_BUTTON (dialed_button), "Out");
     g_signal_connect(
               G_OBJECT(dialed_button),
               "clicked",
@@ -776,7 +1100,7 @@ int main( int argc, char* argv[] )
     gtk_toggle_button_set_mode (GTK_TOGGLE_BUTTON (dialed_button), FALSE);
 
 	recieved_button = hildon_gtk_radio_button_new_from_widget (HILDON_SIZE_AUTO, GTK_RADIO_BUTTON(dialed_button));
-	gtk_button_set_label (GTK_BUTTON (recieved_button), "Received");
+	gtk_button_set_label (GTK_BUTTON (recieved_button), "In");
     g_signal_connect(
               G_OBJECT(recieved_button),
               "clicked",
@@ -805,6 +1129,16 @@ int main( int argc, char* argv[] )
     hildon_app_menu_append (menu, GTK_BUTTON (call_type_button));
     gtk_widget_show(call_type_button);
 
+
+    date_button = gtk_button_new_with_label("Date");
+    g_signal_connect(
+              G_OBJECT(date_button),
+              "clicked",
+              G_CALLBACK(date_dialog),
+              &appdata);
+    hildon_app_menu_append (menu, GTK_BUTTON (date_button));
+    gtk_widget_show(date_button);
+
     gtk_widget_show_all (GTK_WIDGET (menu));
     hildon_window_set_app_menu (HILDON_WINDOW (appdata.mainWindow), menu);
 
